__version__ = '0.5.0'

from .event import Event
from .platform_mock_api_factory import connect
from .util.topic_util import name_for_service_specific_topic
from .util.topic_util import name_for_shared_topic
